/**
  * @file RootGrowth.cpp
  * @author ...
  * ...
  * @@tag DifferenceEquationMultiple@vle.extension.difference-equation @@namespace:tp3_2_correction;class:RootGrowth;par:maxDepth,1300|growthRate,0.40000000000000002|;sync:ThermTime|;nosync:@@end tag@@
  */

#include <vle/extension/DifferenceEquation.hpp>

//@@begin:includes@@
//@@end:includes@@

namespace vd = vle::devs;
namespace ve = vle::extension;
namespace vv = vle::value;

namespace tp3_2_correction {

class RootGrowth : public ve::DifferenceEquation::Multiple
{
public:
    RootGrowth(
       const vd::DynamicsInit& atom,
       const vd::InitEventList& evts)
        : ve::DifferenceEquation::Multiple(atom, evts)
    {
        maxDepth = vv::toDouble(evts.get("maxDepth"));
        growthRate = vv::toDouble(evts.get("growthRate"));
        RootDepth = createVar("RootDepth");
        RootDepthDelta = createVar("RootDepthDelta");
        ThermTime = createSync("ThermTime");
    }

    virtual ~RootGrowth()
    {}

//@@begin:compute@@
virtual void compute(const vd::Time& /*time*/)
{
  RootDepthDelta = growthRate * (ThermTime() - ThermTime(-1));
  RootDepth = std::min(RootDepth(-1) + RootDepthDelta(), maxDepth);

}
//@@end:compute@@

//@@begin:initValue@@
virtual void initValue(const vd::Time& /*time*/)
{ }
//@@end:initValue@@

private:
//@@begin:user@@
//@@end:user@@

    double maxDepth;
    double growthRate;
    Var RootDepth;
    Var RootDepthDelta;
    Sync ThermTime;
};

} // namespace tp3_2_correction

DECLARE_DYNAMICS(tp3_2_correction::RootGrowth)

